/*
  "Brochage" de la carte:
                                      +-----+
         +----[PWR]-------------------| USB |--+
         |                            +-----+  |
         |         GND/RST2  [ ][ ]            |
         |       MOSI2/SCK2  [ ][ ]  A5/SCL[ ] |    
         |          5V/MISO2 [ ][ ]  A4/SDA[ ] |   
         |                             AREF[ ] |
         |                              GND[ ] |
         | [ ]N/C                    SCK/13[A] |   DIRB (Arduino Motor Shield)
         | [ ]IOREF                 MISO/12[A] |   DIRA (Arduino Motor Shield)
         | [ ]RST                   MOSI/11[A]~|   PWMB (SPEEDB) (Arduino Motor Shield)
         | [ ]3V3    +---+               10[ ]~|   
         | [ ]5v    -| A |-               9[ ]~|   
         | [ ]GND   -| R |-               8[ ] |   
         | [ ]GND   -| D |-                    |
         | [ ]Vin   -| U |-               7[ ] |   
         |          -| I |-               6[ ]~|   
         | [ ]A0    -| N |-               5[I]~|   MOTOR 2 ENCODER (Capteur effet Hall)
         | [ ]A1    -| O |-               4[I] |   MOTOR 1 ENCODER (Capteur effet Hall)
         | [ ]A2     +---+           INT1/3[A]~|   PWMA (SPEEDA) (Arduino Motor Shield)
         | [ ]A3                     INT0/2[ ] |   
         | [ ]A4/SDA  RST SCK MISO     TX>1[ ] |   
         | [ ]A5/SCL  [ ] [ ] [ ]      RX<0[ ] |   
         |            [ ] [ ] [ ]              |
         |  UNO_R3    GND MOSI 5V  ____________/
          \_______________________/

  ------------------------------------------------------------------
  Arduino Motor Shield:
  
  Function        pins per Ch. A	pins per Ch. B
  Direction	      D12                 D13
  PWM	            D3                  D11
  Brake	          D9                  D8
  Current Sensing	A0                  A1


  Operating Voltage	5V to 12V (DO NOT USE >9V if powering the Arduino via the Shield VIN)
  Motor controller	L298P, Drives 2 DC motors or 1 stepper motor
  Max current	      2A per channel or 4A max (with external power supply)
  Current sensing	  1.65V/A
  
  Free running stop and brake function Brushed DC motor. 
  You can drive two Brushed DC motors by connecting the two
  wires of each one in the (+) and (-) screw terminals for each channel A and B. 
  In this way you can control its direction by setting HIGH or LOW the DIR A 
  and DIR B pins, you can control the speed by varying the PWM A and PWM B 
  duty cycle values. The Brake A and Brake B pins, if set HIGH, will effectively 
  brake the DC motors rather than let them slow down by cutting the power. 
  You can measure the current going through the DC motor by reading the SNS0 
  and SNS1 pins. On each channel will be a voltage proportional to the measured
  current, which can be read as a normal analog input, through the function 
  analogRead() on the analog input A0 and A1. For your convenience it is 
  calibrated to be 3.3V when the channel is delivering its maximum possible 
  current, that is 2A.

  ------------------------------------------------------------------
  Les broches comme source d'interruption:
  
  Board	Digital Pins Usable for External Interrupts Request: 
  Uno, Nano, Mini, other 328-based: Broches	2 et 3

  Pin Change Interrupt Request:
  Pin Change Interrupt Request 0 (pins D8 to D13) (PCINT0_vect) PCMSK0
  Pin Change Interrupt Request 1 (pins A0 to A5)  (PCINT1_vect) PCMSK1
  Pin Change Interrupt Request 2 (pins D0 to D7)  (PCINT2_vect) PCMSK2

  D0    PCINT16 (PCMSK2 / PCIF2 / PCIE2)
  D1    PCINT17 (PCMSK2 / PCIF2 / PCIE2)
  D2    PCINT18 (PCMSK2 / PCIF2 / PCIE2)
  D3    PCINT19 (PCMSK2 / PCIF2 / PCIE2)
  D4    PCINT20 (PCMSK2 / PCIF2 / PCIE2)
  D5    PCINT21 (PCMSK2 / PCIF2 / PCIE2)
  D6    PCINT22 (PCMSK2 / PCIF2 / PCIE2)
  D7    PCINT23 (PCMSK2 / PCIF2 / PCIE2)
  D8    PCINT0  (PCMSK0 / PCIF0 / PCIE0)
  D9    PCINT1  (PCMSK0 / PCIF0 / PCIE0)
  D10   PCINT2  (PCMSK0 / PCIF0 / PCIE0)
  D11   PCINT3  (PCMSK0 / PCIF0 / PCIE0)
  D12   PCINT4  (PCMSK0 / PCIF0 / PCIE0)
  D13   PCINT5  (PCMSK0 / PCIF0 / PCIE0)
  A0    PCINT8  (PCMSK1 / PCIF1 / PCIE1)
  A1    PCINT9  (PCMSK1 / PCIF1 / PCIE1)
  A2    PCINT10 (PCMSK1 / PCIF1 / PCIE1)
  A3    PCINT11 (PCMSK1 / PCIF1 / PCIE1)
  A4    PCINT12 (PCMSK1 / PCIF1 / PCIE1)
  A5    PCINT13 (PCMSK1 / PCIF1 / PCIE1)

  pin change interrupt example for D9
  PCMSK0 |= bit (PCINT1);  // want pin 9
  PCIFR  |= bit (PCIF0);   // clear any outstanding interrupts
  PCICR  |= bit (PCIE0);   // enable pin change interrupts for D8 to D13

  Références:
  - ATmega328 datasheet Chapitre 17
  - http://playground.arduino.cc/Main/PinChangeInterrupt
  - http://gammon.com.au/interrupts

  ------------------------------------------------------------------
*/

//------------------------------------------------------------------ 
// [A]    Arduino Motor Shield
//------------------------------------------------------------------

#define DIR_A   12
#define SPEED_A 3
#define DIR_B   13
#define SPEED_B 9

//------------------------------------------------------------------ 
// [I]    Speed encoder interrupt
//------------------------------------------------------------------

#define ENCODER1IRQ 4
#define ENCODER2IRQ 5

// conserver le qualificatif "volatile" pour les variables modifiees dans les ISR.
volatile long motor1_count = 0;
volatile long motor2_count = 0;

void setup()
{
  pinMode(DIR_A, OUTPUT);
  pinMode(DIR_B, OUTPUT);
  digitalWrite(SPEED_A,0);
  pinMode(SPEED_A, OUTPUT);
  digitalWrite(SPEED_B,0);
  pinMode(SPEED_B, OUTPUT);

  // Setup encoder ISR
  pinMode(ENCODER1IRQ, INPUT);
  pinMode(ENCODER2IRQ, INPUT);
  
  // On utiliser Pin Change Interrupt au lieu de int.0 et int.1 car lar broche D3 (int.1) est
  // occupée par SPEEDA
  // D4    PCINT20 (PCMSK2 / PCIF2 / PCIE2)
  // D5    PCINT21 (PCMSK2 / PCIF2 / PCIE2)
  PCMSK2 |= bit (PCINT20) | bit(PCINT21);  // D4 et D5
  PCIFR  |= bit (PCIF2);   // clear any outstanding interrupts
  PCICR  |= bit (PCIE2);   // enable pin change interrupts for D0 to D7

  Serial.begin(9600);

  analogWrite(SPEED_A, 150);
}

// Le contenu ci-dessous de loop() correspond à un test
// Ce n'est pas du code "utile"
void loop()
{
  delay(5000);
  String s = F("Mx count = ");
  s[1] = '1';
  Serial.print(s);
  Serial.println(motor1_count);

  s[1] = '2';
  Serial.print(s);
  Serial.println(motor2_count);
  while(1); 
}


// ISR pour les encodeurs
ISR (PCINT2_vect) 
{
  int x = PIND;  // Lecture du port D en entier en une fois
  
  /* Ce code devrait fonctionner aussi bien avec les capteurs qui passent de 0 à 1 que ceux 
   * qui passent de 1 à 0:
   * - de 0 à 1 détecte le FRONT MONTANT initial (quand le capteur "capte") 
   * - de 1 à 0 détecte le FRONT MONTANT final au lieu du FRONT DESCENDANT initial  
   *   (quand le capteur **cesse** de "capter") 
   */
   
  motor1_count += bitRead(x, ENCODER1IRQ); // Attention: dépend de la correspondance entre
  motor2_count += bitRead(x, ENCODER2IRQ); // N° de PIN Arduino et N° de PIN AVR

}

void reset_counters()
{
  // Remettre à 0 les compteurs
  noInterrupts ();
  motor1_count = 0;
  motor2_count = 0;
  interrupts ();

}
